@@ -1,6 +1,11 @@
-from typing import Any
+from typing import Any, Tuple
 
 import cloudpickle
+import numpy as np
+
+gym_old_venv_step_type = Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]
+gym_new_venv_step_type = Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray,
+                               np.ndarray]
 
 
 class CloudpickleWrapper(object):
