@@ -1,10 +1,12 @@
-import warnings
 from abc import ABC, abstractmethod
 from typing import Any, Callable, List, Optional, Tuple, Union
 
 import gym
 import numpy as np
 
+from tianshou.env.utils import gym_new_venv_step_type, gym_old_venv_step_type
+from tianshou.utils import deprecation
+
 
 class EnvWorker(ABC):
     """An abstract worker for an environment."""
@@ -12,8 +14,8 @@ class EnvWorker(ABC):
     def __init__(self, env_fn: Callable[[], gym.Env]) -> None:
         self._env_fn = env_fn
         self.is_closed = False
-        self.result: Union[Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray],
-                           np.ndarray]
+        self.result: Union[gym_old_venv_step_type, gym_new_venv_step_type,
+                           Tuple[np.ndarray, dict], np.ndarray]
         self.action_space = self.get_env_attr("action_space")  # noqa: B009
         self.is_reset = False
 
@@ -33,7 +35,7 @@ class EnvWorker(ABC):
         function is determined by such kind of different signal.
         """
         if hasattr(self, "send_action"):
-            warnings.warn(
+            deprecation(
                 "send_action will soon be deprecated. "
                 "Please use send and recv for your own EnvWorker."
             )
@@ -42,33 +44,35 @@ class EnvWorker(ABC):
                 self.result = self.reset()
             else:
                 self.is_reset = False
-                self.send_action(action)  # type: ignore
+                self.send_action(action)
 
     def recv(
         self
-    ) -> Union[Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray], np.ndarray]:
+    ) -> Union[gym_old_venv_step_type, gym_new_venv_step_type, Tuple[np.ndarray, dict],
+               np.ndarray]:  # noqa:E125
         """Receive result from low-level worker.
 
         If the last "send" function sends a NULL action, it only returns a
         single observation; otherwise it returns a tuple of (obs, rew, done,
-        info).
+        info) or (obs, rew, terminated, truncated, info), based on whether
+        the environment is using the old step API or the new one.
         """
         if hasattr(self, "get_result"):
-            warnings.warn(
+            deprecation(
                 "get_result will soon be deprecated. "
                 "Please use send and recv for your own EnvWorker."
             )
             if not self.is_reset:
-                self.result = self.get_result()  # type: ignore
+                self.result = self.get_result()
         return self.result
 
-    def reset(self) -> np.ndarray:
-        self.send(None)
-        return self.recv()  # type: ignore
+    @abstractmethod
+    def reset(self, **kwargs: Any) -> Union[np.ndarray, Tuple[np.ndarray, dict]]:
+        pass
 
     def step(
         self, action: np.ndarray
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Union[gym_old_venv_step_type, gym_new_venv_step_type]:
         """Perform one timestep of the environment's dynamic.
 
         "send" and "recv" are coupled in sync simulation, so users only call
