@@ -1,5 +1,6 @@
 from typing import (
     Any,
+    Callable,
     Dict,
     List,
     Optional,
@@ -14,6 +15,8 @@ import numpy as np
 import torch
 from torch import nn
 
+from tianshou.data.batch import Batch
+
 ModuleType = Type[nn.Module]
 
 
@@ -262,7 +265,7 @@ class Recurrent(nn.Module):
         """
         obs = torch.as_tensor(
             obs,
-            device=self.device,  # type: ignore
+            device=self.device,
             dtype=torch.float32,
         )
         # obs [bsz, len, dim] (training) or [bsz, dim] (evaluation)
@@ -453,3 +456,61 @@ class BranchingNet(nn.Module):
         action_scores = action_scores - torch.mean(action_scores, 2, keepdim=True)
         logits = value_out + action_scores
         return logits, state
+
+
+def get_dict_state_decorator(
+    state_shape: Dict[str, Union[int, Sequence[int]]], keys: Sequence[str]
+) -> Tuple[Callable, int]:
+    """A helper function to make Net or equivalent classes (e.g. Actor, Critic) \
+    applicable to dict state.
+
+    The first return item, ``decorator_fn``, will alter the implementation of forward
+    function of the given class by preprocessing the observation. The preprocessing is
+    basically flatten the observation and concatenate them based on the ``keys`` order.
+    The batch dimension is preserved if presented. The result observation shape will
+    be equal to ``new_state_shape``, the second return item.
+
+    :param state_shape: A dictionary indicating each state's shape
+    :param keys: A list of state's keys. The flatten observation will be according to \
+    this list order.
+    :returns: a 2-items tuple ``decorator_fn`` and ``new_state_shape``
+    """
+    original_shape = state_shape
+    flat_state_shapes = []
+    for k in keys:
+        flat_state_shapes.append(int(np.prod(state_shape[k])))
+    new_state_shape = sum(flat_state_shapes)
+
+    def preprocess_obs(
+        obs: Union[Batch, dict, torch.Tensor, np.ndarray]
+    ) -> torch.Tensor:
+        if isinstance(obs, dict) or (isinstance(obs, Batch) and keys[0] in obs):
+            if original_shape[keys[0]] == obs[keys[0]].shape:
+                # No batch dim
+                new_obs = torch.Tensor([obs[k] for k in keys]).flatten()
+                # new_obs = torch.Tensor([obs[k] for k in keys]).reshape(1, -1)
+            else:
+                bsz = obs[keys[0]].shape[0]
+                new_obs = torch.cat(
+                    [torch.Tensor(obs[k].reshape(bsz, -1)) for k in keys], dim=1
+                )
+        else:
+            new_obs = torch.Tensor(obs)
+        return new_obs
+
+    @no_type_check
+    def decorator_fn(net_class):
+
+        class new_net_class(net_class):
+
+            def forward(
+                self,
+                obs: Union[np.ndarray, torch.Tensor],
+                *args,
+                **kwargs,
+            ) -> Any:
+                return super().forward(preprocess_obs(obs), *args, **kwargs)
+
+        return new_net_class
+
+    return decorator_fn, new_state_shape
